@@ -1,6 +1,6 @@
-#' @title A function for computing orthogonal dissimilarity matrices (orthoDiss)
+#' @title A function for computing dissimilarity matrices from orthogonal projections (orthoDiss)
 #' @description
-#' This function computes orthogonal dissimilarities between either observations in a given set or between observations in two different sets.
+#' This function computes dissimilarities (in an orthogonal space) between either observations in a given set or between observations in two different sets.
 #' The dissimilarities are computed based on either principal component projection or partial least squares projection of the data. After projecting the data, 
 #' the Mahalanobis distance is applied.
 #' @usage 
@@ -10,7 +10,7 @@
 #'           method = "pca", 
 #'           local = FALSE, 
 #'           k0, 
-#'           center = TRUE, scaled = TRUE, 
+#'           center = TRUE, scaled = FALSE, 
 #'           return.all = FALSE, cores = 1, ...)
 #' @param Xr a \code{matrix} (or \code{data.frame}) containing the (reference) data.
 #' @param X2 an optional \code{matrix} (or \code{data.frame}) containing data of a second set of observations(samples).
@@ -24,7 +24,7 @@
 #' @param method the method for projecting the data. Options are: "pca" (principal component analysis using the singular value decomposition algorithm), "pca.nipals" (principal component analysis using the non-linear iterative partial least squares algorithm) and "pls" (partial least squares). See the \code{\link{orthoProjection}} function for further details on the projection methods.
 #' @param local a logical indicating whether or not to compute the distances locally (i.e. projecting locally the data) by using the \eqn{k0} nearest neighbour samples of each sample. Default is \code{FALSE}. See details.
 #' @param k0 if \code{local = TRUE} a numeric integer value which indicates the number of nearest neighbours(\eqn{k0}) to retain in order to recompute the local orthogonal distances.
-#' @param center a logical indicating if the spectral data \code{Xr} (and \code{X2} if specified) must be centered. If \code{X2} is specified the data is scaled on the basis of \eqn{Xr \cup Xu}.
+#' @param center a logical indicating if the spectral data \code{Xr} (and \code{X2} if specified) must be centered. If \code{X2} is specified the data is centered on the basis of \eqn{Xr \cup Xu}. For dissimilarity computations based on pls, the data is always centered for the projections. 
 #' @param scaled a logical indicating if \code{Xr} (and \code{X2} if specified) must be scaled. If \code{X2} is specified the data is scaled on the basis of \eqn{Xr \cup Xu}.
 #' @param return.all a logical. In case \code{X2} is specified it indicates whether or not the distances between all the elements resulting from \eqn{Xr \cup Xu} must be computed.
 #' @param cores number of cores used when \code{method} in \code{pcSelection} is \code{"opc"} (which can be computationally intensive) and \code{local = FALSE} (default = 1). Dee details.
@@ -36,6 +36,7 @@
 #' @return a \code{list} of class \code{orthoDiss} with the following components:
 #' \itemize{
 #'  \item{\code{n.components}}{ the number of components (either principal components or partial least squares components) used for computing the global distances.}
+#'  \item{\code{global.variance.info}}{ the information about the expalined variance(s) of the projection. When \code{local = TRUE}, the information corresponds to the global projection done prior computing the local projections.}
 #'  \item{\code{loc.n.components}}{ if \code{local = TRUE}, a \code{data.frame} which specifies the number of local components (either principal components or partial least squares components) used for computing the dissimilarity between each target sample and its neighbour samples.}
 #'  \item{\code{dissimilarity}}{ the computed dissimilarity matrix. If \code{local = FALSE} a distance \code{matrix}. If \code{local = TRUE} a \code{matrix} of class \code{orthoDiss}. In this case each column represent the dissimilarity between a target sample and its neighbourhood.}
 #'  }
@@ -114,6 +115,10 @@
 ## 09.03.2014 Leo     In the doc was specified that multi-threading is 
 ##                    not working for mac
 ## 13.03.2014 Antoine The explanation of the cores argument was modified
+## 07.09.2014 Antoine A bug handling Yr as a matrix was fixed
+## 02.12.2015 Leo     The function now outputs an the object global.variance.info
+##                    which provides information on the explained variance
+##                    of the projections.
 
 orthoDiss <- function(Xr, X2 = NULL, 
                       Yr = NULL, 
@@ -121,7 +126,7 @@ orthoDiss <- function(Xr, X2 = NULL,
                       method = "pca", 
                       local = FALSE, 
                       k0, 
-                      center = TRUE, scaled = TRUE, 
+                      center = TRUE, scaled = FALSE, 
                       return.all = FALSE, cores = 1, ...){
    
   in.call <- match.call()
@@ -154,8 +159,13 @@ orthoDiss <- function(Xr, X2 = NULL,
   
   if(!is.null(Yr))
   {
-    Yr <- matrix(Yr,length(Yr))
+    if(is.null(dim(Yr)))
+      Yr <- matrix(Yr, length(Yr))
     Yr <- as.data.frame(Yr)
+  }else{
+    if(pcSelection[[1]] == "opc" | method == "pls"){
+      stop("Yu must be provided either when the 'opc' is used in pcSelection is used or method = 'pls'", call. = call.)
+    }
   }
   
   prj <- orthoProjection(Xr = Xr, Yr = Yr, X2 = X2, method = method, pcSelection = pcSelection, center = center, scaled = scaled, cores = cores, call. = FALSE)
@@ -308,14 +318,14 @@ orthoDiss <- function(Xr, X2 = NULL,
         
       }
     }
-    resultsList <- list(n.components = n.components, loc.n.components = data.frame(sample.nm = colnames(distnc), sample = 1:ncol(distnc), loc.n.components = loc.n.components), dissimilarity = distnc)
+    resultsList <- list(n.components = n.components, global.variance.info = prj$variance, loc.n.components = data.frame(sample.nm = colnames(distnc), sample = 1:ncol(distnc), loc.n.components = loc.n.components), dissimilarity = distnc)
     class(resultsList) <- c("orthoDiss", "list") 
-    class(resultsList$dissimilarity) <- c("localOrthoDiss")
+    class(resultsList$dissimilarity) <- c("localOrthoDiss","matrix")
     return(resultsList)    
   }else{
-    resultsList <- list(n.components = n.components, dissimilarity = distnc)
+    resultsList <- list(n.components = n.components, global.variance.info = prj$variance, dissimilarity = distnc)
     class(resultsList) <- c("orthoDiss", "list") 
-    class(resultsList$dissimilarity) <- c("orthoDiss") 
+    class(resultsList$dissimilarity) <- c("orthoDiss","matrix") 
     return(resultsList)
   }
 }
